"""Quvi is a media url parser supporting several hosts (youtube, dailymotion,
vimeo...)
See http://quvi.sourceforge.net/

You can use this module to parse an url from youtube, dailymotion... like this:
>> import quvi
>> q = Quvi()
>> q.parse("http://....")
>> print q.get_properties()

"""
__author__="Patrice FERLET <metal3d@gmail.com>", "Raphaël VINOT <raphael.vinot@gmail.com>"
__license__="LGPLv2.1+."

cimport cquvi

cdef class Quvi:
    #those handles ctypes from quvi.h
    cdef cquvi.quvi_t _c_quvi
    cdef cquvi.quvi_media_t _c_m

    def __cinit__ (self):
        """Initialize quvi handle"""
        cquvi.quvi_init(&self._c_quvi)
        self._c_m = NULL

    cdef cquvi.QUVIcode _c_parse(self, char* url):
        """Parses given url parameters"""
        rc = cquvi.quvi_parse(self._c_quvi, url, &self._c_m)
        return rc

    cdef cquvi.QUVIcode _c_setopt(self, cquvi.QUVIoption option_id, \
            void* parameter):
        """Set an option"""
        rc = cquvi.quvi_setopt(self._c_quvi, option_id, parameter)
        return rc

    def get_error_message(self, code):
        """Get the error string from the library"""
        return cquvi.quvi_strerror(self._c_quvi, code)

    def get_version(self):
        """Get the version(s), only QUVI_VERSION is known by the C library"""
        version = cquvi.quvi_version(cquvi.QUVI_VERSION)
        return version

    def get_supported_ident_properties(self, char* url):
        """Get a few properties for an URL, does not needs connectivity."""
        cdef char* resc = NULL
        cdef cquvi.quvi_ident_t _c_ident = NULL
        rc = cquvi.quvi_supported_ident(self._c_quvi, url, &_c_ident)

        if rc != cquvi.QUVI_OK:
            cquvi.quvi_supported_ident_close(&_c_ident)
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.get_response_code.__name__, rc, msg)
            raise error
        else:
            res = {}

            cquvi.quvi_ident_getprop(_c_ident, \
                    cquvi.QUVI_IDENT_PROPERTY_URL, &resc)
            res['url'] = resc

            cquvi.quvi_ident_getprop(_c_ident, \
                    cquvi.QUVI_IDENT_PROPERTY_DOMAIN, &resc)
            res['domain'] = resc

            cquvi.quvi_ident_getprop(_c_ident, \
                    cquvi.QUVI_IDENT_PROPERTY_FORMATS, &resc)
            res['formats'] = resc

            #NOTE: cquvi.QUVI_IDENT_PROPERTY_CATEGORIES does not work,
            #      segfault
            #cquvi.quvi_ident_getprop(self._c_ident, \
            #        cquvi.QUVI_IDENT_PROPERTY_CATEGORIES, &resc)
            #res['categories'] = resc

            cquvi.quvi_supported_ident_close(&_c_ident)

            return res


    def is_supported(self, char* url):
        """Is the URL supported ? True/False/Unknown"""

        rc = cquvi.quvi_supported(self._c_quvi, url)
        if rc == cquvi.QUVI_OK:
            return True
        elif rc == cquvi.QUVI_NOSUPPORT:
            return False
        else:
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.get_response_code.__name__, rc, msg)
            raise error

    def _get_info(self, info):
        """Get a particular info after trying to fetch properties"""
        cdef long _c_session_info = -1
        rc = cquvi.quvi_getinfo(self._c_quvi, info, &_c_session_info)
        if rc != cquvi.QUVI_OK:
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.get_response_code.__name__, rc, msg)
            raise error
        else:
            return _c_session_info

    def get_curl_session_handle(self):
        """Get the latest curl session handle"""
        return self._get_info(cquvi.QUVIINFO_CURL)

    def get_response_code(self):
        """Get the latest HTTP response code"""
        return self._get_info(cquvi.QUVIINFO_RESPONSECODE)

    def get_formats(self, char* url):
        """Return an array with the available formats"""
        cdef char *_c_formats = NULL
        rc = cquvi.quvi_query_formats(self._c_quvi, url, &_c_formats)
        if rc != cquvi.QUVI_OK:
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.get_formats.__name__, rc, msg)
            raise error
        else:
            return _c_formats.split("|")

    def set_format(self, char* c_format):
        """Set a format

        :param c_format: format to use (ex.: fmt45_720p)
        """
        rc = self._c_setopt(cquvi.QUVIOPT_FORMAT, c_format)
        if rc != cquvi.QUVI_OK:
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.set_format.__name__, rc, msg)
            raise error

    def parse(self, char *url):
        """Parses given url parameters

        :param url: media webpage url (in form http://...)
        """
        rc = self._c_parse(url)
        if rc != cquvi.QUVI_OK:
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.parse.__name__, rc, msg)
            raise error

    def get_properties(self):
        """Returns a dict with media properties

        returned properties are:
            - hostid
            - pageurl
            - pagetitle
            - mediaid
            - mediaurl
            - mediacontentlength
            - mediacontenttype
            - filesuffix
            - responsecode
            - format
            - starttime
            - mediathumbnail
            - mediaduration

        """
        res = {}
        cdef char* resc = NULL
        cdef int   resi = -1
        cdef long resl = -1
        cdef double resd = -1

        if self._c_m is NULL:
            error = QuviError()
            error.custom_error("Parse an url before trying to get the properties")
            raise error

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_HOSTID, &resc)
        res['hostid'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_PAGEURL, &resc)
        res['pageurl'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_PAGETITLE, &resc)
        res['pagetitle'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_MEDIAID, &resc)
        res['mediaid'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_MEDIAURL, &resc)
        res['mediaurl'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_MEDIACONTENTLENGTH, &resl)
        res['mediacontentlength'] = resl;

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_MEDIACONTENTTYPE, &resc)
        res['mediacontenttype'] = resc;

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_FILESUFFIX, &resc)
        res['filesuffix'] = resc;

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_RESPONSECODE, &resl)
        res['responsecode'] = resl

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_FORMAT, &resc)
        res['format'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_STARTTIME, &resc)
        res['starttime'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_MEDIATHUMBNAILURL, &resc)
        res['mediathumbnail'] = resc

        cquvi.quvi_getprop(self._c_m, cquvi.QUVIPROP_MEDIADURATION, &resd)
        res['mediaduration'] = resd

        return res

    def nextmediaurl(self):
        """Jumps to next media

        :returns Boolean True if other media is found, False either
        Raise an exception on error"""

        rc = cquvi.quvi_next_media_url(self._c_m)

        if rc == cquvi.QUVI_LAST:
            return False

        if rc != cquvi.QUVI_OK:
            error = QuviError()
            msg = self.get_error_message(rc)
            error.return_code_error(self.nextmediaurl.__name__, rc, msg)
            raise error

        return True

    def __del__(self):
        """Cleanup media handles"""
        cquvi.quvi_parse_close(&self._c_m);
        cquvi.quvi_close(&self._c_quvi)
        cquvi.quvi_free(&self._c_quvi)

def get_properties_best_quality(url):
    """Get the properties of the best available format of a video"""
    q = Quvi()
    if q.is_supported(url):
        formats = q.get_formats(url)
        best_format = formats[-1]
        q.set_format(best_format)
        q.parse(url)
        properties = q.get_properties()
        return properties
    return None

class QuviError(Exception):
    """Exception to raise on QuviErrors"""
    def __init__(self):
        self.error_msg = ""

    def custom_error(self, msg):
        """Set a custom error message"""
        self.error_msg = "An error occurred: " + msg

    def return_code_error(self, function_name, value, msg):
        """Set an error message defined by the C library"""
        self.error_msg = "An error occurred in function \"{fct}\": {detail} ({code})"\
                .format(fct = function_name, detail = msg, code = value)

    def __str__(self):
        return repr(self.error_msg)



"""
This is the pxd file used to define needed functions/types from
libquvi
"""
__author__="Patrice FERLET <metal3d@gmail.com>"
__license__="LGPLv2.1+."

cdef extern from "quvi/quvi.h":

    ctypedef struct quvi_t:
        pass

    ctypedef void* quvi_media_t
    ctypedef void* quvi_video_t
    ctypedef void* quvi_ident_t
    ctypedef void* quvi_callback_status
    ctypedef enum QUVIversion:
        QUVI_VERSION

    ctypedef enum QUVIcode:
        QUVI_OK  = 0x00
        QUVI_MEM
        QUVI_BADHANDLE
        QUVI_INVARG
        QUVI_CURLINIT
        QUVI_LAST
        QUVI_ABORTEDBYCALLBACK
        QUVI_LUAINIT
        QUVI_NOLUAWEBSITE
        QUVI_NOLUAUTIL
        _INTERNAL_QUVI_LAST

        QUVI_NOSUPPORT
        QUVI_CALLBACK
        QUVI_ICONV
        QUVI_LUA

    ctypedef enum QUVIstatus:
        QUVISTATUS_FETCH
        QUVISTATUS_VERIFY
        QUVISTATUS_RESOLVE

    ctypedef enum QUVIstatusType:
        QUVISTATUSTYPE_PAGE
        QUVISTATUSTYPE_CONFIG
        QUVISTATUSTYPE_PLAYLIST
        QUVISTATUSTYPE_DONE

    ctypedef enum QUVIoption:
        QUVIOPT_FORMAT
        QUVIOPT_NOVERIFY
        QUVIOPT_STATUSFUNCTION
        QUVIOPT_NORESOLVE
        QUVIOPT_CATEGORY
        QUVIOPT_FETCHFUNCTION
        QUVIOPT_RESOLVEFUNCTION
        QUVIOPT_VERIFYFUNCTION

    ctypedef enum QUVIcategory:
        QUVIPROTO_HTTP
        QUVIPROTO_MMS
        QUVIPROTO_RTSP
        QUVIPROTO_RTMP
        QUVIPROTO_ALL

    ctypedef enum QUVIinfo:
        QUVIINFO_NONE
        QUVIINFO_CURL
        QUVIINFO_RESPONSECODE

    ctypedef enum QUVIproperty:
        QUVIPROP_NONE
        QUVIPROP_HOSTID
        QUVIPROP_PAGEURL
        QUVIPROP_PAGETITLE
        QUVIPROP_MEDIAID
        QUVIPROP_MEDIAURL
        QUVIPROP_MEDIACONTENTLENGTH
        QUVIPROP_MEDIACONTENTTYPE
        QUVIPROP_FILESUFFIX
        QUVIPROP_RESPONSECODE
        QUVIPROP_FORMAT
        QUVIPROP_STARTTIME
        QUVIPROP_MEDIATHUMBNAILURL
        QUVIPROP_MEDIADURATION
        _QUVIPROP_LAST

    ctypedef enum QUVIidentProperty:
        QUVI_IDENT_PROPERTY_URL
        QUVI_IDENT_PROPERTY_DOMAIN
        QUVI_IDENT_PROPERTY_FORMATS

    ctypedef int quvi_word
    ctypedef int quvi_byte

    QUVIcode quvi_init(quvi_t *quvi)
    QUVIcode quvi_getinfo(quvi_t quvi, QUVIinfo info, ...)
    QUVIcode quvi_setopt(quvi_t quvi, QUVIoption opt, ...)
    size_t quvi_write_callback_default(void *ptr, size_t size, size_t nmemb, void *stream)
    void quvi_close(quvi_t* quvi)
    QUVIcode quvi_parse(quvi_t quvi, char *url, quvi_media_t *media)
    QUVIcode quvi_getprop(quvi_media_t media, QUVIproperty prop, ...)
    QUVIcode quvi_next_media_url(quvi_media_t media)
    QUVIcode quvi_next_videolink(quvi_video_t video)
    void quvi_parse_close(quvi_media_t *media)
    QUVIcode quvi_supported(quvi_t quvi, char *url)
    QUVIcode quvi_supported_ident(quvi_t quvi, char *url, quvi_ident_t *ident)
    QUVIcode quvi_ident_getprop(quvi_ident_t handle, QUVIidentProperty property, ...)
    void quvi_supported_ident_close(quvi_ident_t *ident)
    QUVIcode quvi_next_supported_website(quvi_t quvi, char **domain, char **formats)
    QUVIcode quvi_next_host(char **domain, char **formats)
    char *quvi_strerror(quvi_t quvi, QUVIcode code)
    char *quvi_version(QUVIversion type)
    void quvi_free(void *ptr)
    QUVIcode quvi_query_formats(quvi_t session, char *url, char **formats)




from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext

ext_modules = [Extension("quvi", ["src/quvi.pyx"], libraries=['quvi'])]

setup(
  name = 'Quvi',
  author = 'Patrice FERLET',
  author_email = "metal3d@gmail.com",
  url="https://github.com/metal3d/python-libquvi",
  description="libquvi wrapper module",
  license="LGPLv2.1+",
  version="0.0.1-b1",
  cmdclass = {'build_ext': build_ext},
  ext_modules = ext_modules,
  long_description = "".join(open('README.rst').readlines()),

  classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Cython',
        'Topic :: Internet',
        'Topic :: Multimedia :: Video',
  ]
)



#!/usr/bin/env python2
# Copyright 2011 Tom Vincent <http://tlvince.com/contact/>

import quvi

"""Nose test cases for python-quvi."""

def testInvalidURL():
    """Test a non-supported URL."""
    q = quvi.Quvi()
    try:
        q.parse("http://google.com")
    except quvi.QuviError:
        assert True

def testValidURL():
    """Test a known-supported URL."""
    q = quvi.Quvi()
    q.parse("http://www.youtube.com/watch?v=0gzA6Xzbh1k")

    expected = "youtube"
    actual = q.get_properties()["hostid"]

    assert actual == expected
